package core.api.controller;

import core.api.models.auth.SimpleStatusResponse;
import core.api.models.timemanager.taskSchedule.BasicTaskScheduleEntityInfo;
import core.api.models.timemanager.taskSchedule.BasicTaskScheduleFieldInfo;
import core.api.models.timemanager.taskSchedule.ScheduleInfo;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.ScheduleType;
import core.entities.timemanager.Task;
import core.repositories.timemanager.BasicTaskScheduleRepository;
import core.services.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class ScheduleController {

    private final TaskService taskService;
    private final TaskScheduleService taskScheduleService;
    private final BasicTaskScheduleRepository basicTaskScheduleRepository;

    public ScheduleController(@Autowired TaskService taskService, @Autowired TaskScheduleService taskScheduleService,
                              BasicTaskScheduleRepository basicTaskScheduleRepository) {
        this.taskService = taskService;
        this.taskScheduleService = taskScheduleService;
        this.basicTaskScheduleRepository = basicTaskScheduleRepository;
    }

    @GetMapping("/schedules/{taskID}/{scheduleType}")
    public ResponseEntity<?> loadSchedulesOfTask(@PathVariable long taskID, @PathVariable ScheduleType scheduleType) {
        PermissionResult<Task> taskPermissionResult = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!taskPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        switch (scheduleType) {
            case BASIC -> {
                List<BasicTaskSchedule> basicTaskScheduleList = new ArrayList<>(taskPermissionResult.getResult().getBasicTaskSchedules());
                basicTaskScheduleList.sort(Comparator.comparing(BasicTaskSchedule::getScheduleDate));
                return ResponseEntity.ok(basicTaskScheduleList.stream().map(BasicTaskScheduleEntityInfo::new));
            }
            case ADVANCED, MODERATE -> ResponseEntity.ok(new ArrayList<>());
        }
        return null;
    }

    @PutMapping("/schedules/{taskID}/basic")
    public ResponseEntity<?> basicScheduleTask(@PathVariable long taskID, @Valid @RequestBody BasicTaskScheduleFieldInfo basicTaskSchedule) {
        PermissionResult<Task> taskPermissionResult = taskService.getTaskPermissions(taskID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!taskPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        ServiceResult<BasicTaskSchedule> creationResult = taskScheduleService.createBasicTaskSchedule(taskPermissionResult.getResult(), basicTaskSchedule);
        return ResponseEntity.ok(new BasicTaskScheduleEntityInfo(creationResult.getResult()));
    }

    @PostMapping("/schedules/{scheduleID}/basic")
    public ResponseEntity<?> editScheduledTask(@PathVariable long scheduleID, @Valid @RequestBody BasicTaskScheduleFieldInfo basicTaskScheduleFieldInfo) {
        PermissionResult<BasicTaskSchedule> permissionResult = taskScheduleService.getSchedulePermissions(scheduleID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        taskScheduleService.reschedule(permissionResult.getResult(), basicTaskScheduleFieldInfo);
        return ResponseEntity.ok(new SimpleStatusResponse("success"));
    }

    @DeleteMapping("/schedules/{scheduleID}/{scheduleType}")
    public ResponseEntity<?> deleteScheduledTask(@PathVariable long scheduleID, @PathVariable ScheduleType scheduleType) {
        PermissionResult<BasicTaskSchedule> permissionResult = taskScheduleService.getSchedulePermissions(scheduleID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!permissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(permissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        taskScheduleService.deleteBasicSchedule(permissionResult.getResult());
        return ResponseEntity.ok(new SimpleStatusResponse("success"));
    }

    @GetMapping("/schedules/today")
    public ResponseEntity<?> loadTodaysSchedules() {
        ServiceResult<List<BasicTaskSchedule>> todaysSchedules = taskScheduleService.loadTodaysSchedule(SecurityContextHolder.getContext().getAuthentication().getName());
        if(todaysSchedules.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        } else {
            return ResponseEntity.ok(todaysSchedules.getResult().stream().map(ScheduleInfo::new).toList());
        }
    }

    @GetMapping("/schedules")
    public ResponseEntity<?> loadSchedules() {
        ServiceResult<List<BasicTaskSchedule>> schedules = taskScheduleService.loadSchedules(SecurityContextHolder.getContext().getAuthentication().getName());
        if(schedules.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        } else {
            return ResponseEntity.ok(schedules.getResult().stream().map(ScheduleInfo::new).toList());
        }
    }
}
